const {readFileSync} = require('fs');
const path = require('path');
const R = require('ramda');
const fetch = require('node-fetch');

class PublicationInformation {
  constructor(fetchFn = fetch) {
    this._fetchFn = fetchFn;
    this._updatePublicationInformation(JSON.parse(readFileSync(path.resolve(__dirname, 'pubs.json'))));
    this._triggerPublicationMappingRedownload();
    setInterval(() => {
      this._triggerPublicationMappingRedownload();
    }, 1000 * 60 * 10);
  }
  get publicationInformation() {
    return this._publicationInformation;
  }
  async _triggerPublicationMappingRedownload() {
    let result;
    try {
      result = await this._fetchFn('https://reach-dataen-publication-mapping.s3.eu-west-1.amazonaws.com/pubs.json');
      if(!result.ok) throw JSON.stringify(result);
    } catch(err) {
      console.error(`Could not re-download publication information, ${err}`);
      return;
    }
    try {
      this._updatePublicationInformation(JSON.parse(await result.text()));
    } catch(err) {
      console.error(`Could not parse publication information redownload, ${err}`);
      return;
    }
  }
  _updatePublicationInformation(newPublicationInformation) {
    this._publicationInformation = newPublicationInformation.map(pubInfo => {
      pubInfo.url = pubInfo.urls[0];
      return pubInfo;
    });
  }
  findHostname(hostname) {
    if(!hostname) hostname = '';
    if(hostname.endsWith('.trem.media')) hostname = 'trem.media.' + hostname.replace('.trem.media', '');
    if(hostname.endsWith('.translate.goog')) hostname = 'translate.goog.' + hostname.replace('.translate.goog', '').replace(/-/g,'.');
    return R.clone(this.publicationInformation
      .filter(pubInfo => {
        return pubInfo.urls.some(url => hostname.endsWith(url));
      })[0]);
  }
  findName(name) {
    const normalizedName = name.toString().toLowerCase();
    return R.clone(this.publicationInformation
      .filter(pubInfo => {
        return pubInfo.names.some(pubName => pubName.toLocaleLowerCase() === normalizedName);
      })[0]);
  }
  findId(platform, publicationId) {
    return R.clone(this.publicationInformation
      .filter(pubInfo => {
        return pubInfo.platform === platform && pubInfo.publicationId === publicationId;
      })[0]);
  }
  clone() {
    return R.clone(this.publicationInformation);
  }
}

module.exports = PublicationInformation;