const R = require('ramda');
const { URL } = require('url');
const PublicationInformation = require('./PublicationInformation.js');

const publicationInformation = new PublicationInformation();

module.exports = function (inputUrl) {
  let url;
  try {
    url = new URL(inputUrl);
  } catch(e) {
    return; //Nothing more to do here!
  }
  if (!url || !url.hostname || !url.pathname) return;
  const values = publicationInformation.findHostname(url.hostname);
  if (!values) return;
  //eslint doesn't handle regex well
  //eslint-disable-next-line no-useless-escape
  const channelMatch = url.pathname.match(/\/([^\/]+)\//);
  if (channelMatch && channelMatch[1]) {
    values.channel = channelMatch[1];
    //Should always exist given the above
    //eslint-disable-next-line no-useless-escape
    values.section = R.reverse(R.reverse(url.pathname).match(/\/([^\/]+)/)[1]);
    values.fullSection = url.pathname.split('/').slice(1,-1).join('/');
    //reverse here to avoid regex DOS
    const articleIdMatch = R.reverse(url.pathname.replace(/\.amp/, '')).match(/^([0-9]+)-/);
    if (articleIdMatch && articleIdMatch[1]) values.articleId = R.reverse(articleIdMatch[1]);
  }
  return values;
};

module.exports.nameLookUp = name => publicationInformation.findName(name);

module.exports.idLookUp = (platform, id) => publicationInformation.findId(platform, id);

module.exports.publicationInfo = () => {
  return publicationInformation.clone();
};
